package controllers.users;

import ORM.BasketContent;
import ORM.Product;
import ORM.User;
import helpers.Baker;
import helpers.Hasher;
import helpers.Messager;
import helpers.Texter;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.servlet.http.Cookie;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import util.HibernateUtil;

/**
 * Bean to handle user stuff
 * @author Jakub Macoun
 */
@ManagedBean(name = "user")
@SessionScoped
public class UserSessionBean implements Serializable {

    private String locale = Locale.getDefault().toString();
    private User loggedUser;
    private User tempUser = new User();
    private boolean autologin;
    private boolean humanValidated = false;
    private Product viewedProduct;
    private String newPassword;
    private int basketItemsCount;
    private List<BasketContent> basketContent;
    private int basketCountSum = 0;
    private double basketPriceSum = 0;
    private Session hibernate;
    
    public static final int MIN_PASSWORD_LENGTH = 5;
    
    /**
     * Creates a new instance of UserSessionBean
     */
    public UserSessionBean() {
        hibernate = HibernateUtil.getSessionFactory().openSession();
    }
    
    /**
     * Perform user login
     */
    public void login() {
        // Already logged in?
        if (loggedUser instanceof User) {
            return;
        }
        
        // Find loggedUser
        Query query = hibernate.createQuery("FROM User WHERE (email = :email OR id = " + tempUser.getId() + ") AND password = :password");
        query.setParameter("email", tempUser.getEmail());
        query.setParameter("password", Hasher.sha512(tempUser.getPassword()));
        loggedUser = (User) query.uniqueResult();
        
        // Check
        if (!isLogged()) {
            Messager.addError(Texter.get("messages", "loginFail"));
            return;
        }
        
        // Remember?
        if (autologin) {
            Baker.createCookie("autologin", loggedUser.getId() + "-" + tempUser.getPassword());
        }
        
        loadBasket();
        tempUser = new User();
    }
    
    /**
     * Validates user as a human with CAPTCHA
     * @return Navigation - stay
     */
    public String validateHuman() {
        humanValidated = true;
        return "/register";
    }
    
    /**
     * Logs out the user
     */
    public void logout() {
        Baker.removeCookie("autologin");
        loggedUser = null;
    }
    
    /**
     * Updates current user data
     */
    public void updateData() {
        Transaction transaction = hibernate.beginTransaction();
        hibernate.update(loggedUser);
        transaction.commit();
    }
    
    /**
     * Changes current user password
     */
    public void changePassword() {
        newPassword = Hasher.sha512(newPassword);
        
        // Check
        if (!loggedUser.getPassword().equals(newPassword)) {
            Messager.addError("user_password:password_old", Texter.get("messages", "wrongOldPassword"));
            return;
        }
        
        // Change
        loggedUser.setPassword(newPassword);
        updateData();
        Messager.addInfo("Vaše heslo bylo změněno");
    }

    /**
     * @return the loggedUser
     */
    public User getLoggedUser() {
        return loggedUser;
    }

    /**
     * @param loggedUser the loggedUser to set
     */
    public void setLoggedUser(User loggedUser) {
        this.loggedUser = loggedUser;
    }

    /**
     * @return the tempUser
     */
    public User getTempUser() {
        return tempUser;
    }

    /**
     * @param tempUser the tempUser to set
     */
    public void setTempUser(User tempUser) {
        this.tempUser = tempUser;
    }
    
    /**
     * @return the MIN_PASSWORD_LENGTH
     */
    public int getMIN_PASSWORD_LENGTH() {
        return MIN_PASSWORD_LENGTH;
    }
    
    /**
     * Is a user logged?
     * @return True if user is logged in
     */
    public boolean isLogged() {
        return loggedUser instanceof User;
    }

    /**
     * @return the autologin
     */
    public boolean isAutologin() {
        return autologin;
    }

    /**
     * @param autologin the autologin to set
     */
    public void setAutologin(boolean autologin) {
        this.autologin = autologin;
    }
    
    
    /**
     * Checks if it is possible to autologin and does so
     */
    @PostConstruct
    private void doAutologin() {
        Cookie loginCookie = Baker.getCookie("autologin");
        if (!isLogged() && loginCookie != null) {
            String[] loginData = loginCookie.getValue().split("-");
            tempUser.setId(Long.valueOf(loginData[0]));
            tempUser.setPassword(loginData[1]);
            
            login();
        }
    }
    
    /**
     * Loads basket basic info
     */
    public void loadBasket() {
        basketContent = new ArrayList<BasketContent>();
        basketCountSum = 0;
        basketPriceSum = 0;
        
        try {
            basketItemsCount = loggedUser.getBasket().getContent().size();
            
            for (BasketContent content : loggedUser.getBasket().getContent()) {
                basketContent.add(content);
                
                basketCountSum += content.getCount();
                basketPriceSum += content.getProduct().getPrice() * content.getCount();
            }
        } catch (NullPointerException ex) {
            basketItemsCount = 0;
        }
    }

    /**
     * @return the humanValidated
     */
    public boolean isHumanValidated() {
        return isLogged() || humanValidated;
    }

    /**
     * @param humanValidated the humanValidated to set
     */
    public void setHumanValidated(boolean humanValidated) {
        this.humanValidated = humanValidated;
    }

    /**
     * @return the viewedProduct
     */
    public Product getViewedProduct() {
        return viewedProduct;
    }

    /**
     * @param viewedProduct the viewedProduct to set
     */
    public void setViewedProduct(Product viewedProduct) {
        this.viewedProduct = viewedProduct;
    }

    /**
     * @param newPassword the newPassword to set
     */
    public void setNewPassword(String newPassword) {
        this.newPassword = newPassword;
    }

    /**
     * @return the newPassword
     */
    public String getNewPassword() {
        return newPassword;
    }

    /**
     * @return the locale
     */
    public String getLocale() {
        return locale;
    }

    /**
     * @param locale the locale to set
     */
    public void setLocale(String locale) {
        this.locale = locale;
    }

    /**
     * @return the basketItemsCount
     */
    public int getBasketItemsCount() {
        return basketItemsCount;
    }

    /**
     * @param basketItemsCount the basketItemsCount to set
     */
    public void setBasketItemsCount(int basketItemsCount) {
        this.basketItemsCount = basketItemsCount;
    }

    /**
     * @return the basketContent
     */
    public List<BasketContent> getBasketContent() {
        return basketContent;
    }

    /**
     * @param basketContent the basketContent to set
     */
    public void setBasketContent(List<BasketContent> basketContent) {
        this.basketContent = basketContent;
    }

    /**
     * @return the basketPriceSum
     */
    public double getBasketPriceSum() {
        return basketPriceSum;
    }

    /**
     * @return the basketCountSum
     */
    public int getBasketCountSum() {
        return basketCountSum;
    }

    /**
     * @param basketCountSum the basketCountSum to set
     */
    public void setBasketCountSum(int basketCountSum) {
        this.basketCountSum = basketCountSum;
    }

    /**
     * @param basketPriceSum the basketPriceSum to set
     */
    public void setBasketPriceSum(double basketPriceSum) {
        this.basketPriceSum = basketPriceSum;
    }

    /**
     * @return the hibernate
     */
    public Session getHibernate() {
        return hibernate;
    }
}